@@ -5,53 +5,9 @@
 
 #include "input_events.h"
 #include "screen.h"
+#include "shortcut_mod.h"
 #include "util/log.h"
 
-#define SC_SDL_SHORTCUT_MODS_MASK (KMOD_CTRL | KMOD_ALT | KMOD_GUI)
-
-static inline uint16_t
-to_sdl_mod(uint8_t shortcut_mod) {
-    uint16_t sdl_mod = 0;
-    if (shortcut_mod & SC_SHORTCUT_MOD_LCTRL) {
-        sdl_mod |= KMOD_LCTRL;
-    }
-    if (shortcut_mod & SC_SHORTCUT_MOD_RCTRL) {
-        sdl_mod |= KMOD_RCTRL;
-    }
-    if (shortcut_mod & SC_SHORTCUT_MOD_LALT) {
-        sdl_mod |= KMOD_LALT;
-    }
-    if (shortcut_mod & SC_SHORTCUT_MOD_RALT) {
-        sdl_mod |= KMOD_RALT;
-    }
-    if (shortcut_mod & SC_SHORTCUT_MOD_LSUPER) {
-        sdl_mod |= KMOD_LGUI;
-    }
-    if (shortcut_mod & SC_SHORTCUT_MOD_RSUPER) {
-        sdl_mod |= KMOD_RGUI;
-    }
-    return sdl_mod;
-}
-
-static bool
-is_shortcut_mod(struct sc_input_manager *im, uint16_t sdl_mod) {
-    // keep only the relevant modifier keys
-    sdl_mod &= SC_SDL_SHORTCUT_MODS_MASK;
-
-    // at least one shortcut mod pressed?
-    return sdl_mod & im->sdl_shortcut_mods;
-}
-
-static bool
-is_shortcut_key(struct sc_input_manager *im, SDL_Keycode keycode) {
-    return (im->sdl_shortcut_mods & KMOD_LCTRL && keycode == SDLK_LCTRL)
-        || (im->sdl_shortcut_mods & KMOD_RCTRL && keycode == SDLK_RCTRL)
-        || (im->sdl_shortcut_mods & KMOD_LALT  && keycode == SDLK_LALT)
-        || (im->sdl_shortcut_mods & KMOD_RALT  && keycode == SDLK_RALT)
-        || (im->sdl_shortcut_mods & KMOD_LGUI  && keycode == SDLK_LGUI)
-        || (im->sdl_shortcut_mods & KMOD_RGUI  && keycode == SDLK_RGUI);
-}
-
 void
 sc_input_manager_init(struct sc_input_manager *im,
                       const struct sc_input_manager_params *params) {
@@ -73,7 +29,7 @@ sc_input_manager_init(struct sc_input_manager *im,
     im->legacy_paste = params->legacy_paste;
     im->clipboard_autosync = params->clipboard_autosync;
 
-    im->sdl_shortcut_mods = to_sdl_mod(params->shortcut_mods);
+    im->sdl_shortcut_mods = sc_shortcut_mods_to_sdl(params->shortcut_mods);
 
     im->vfinger_down = false;
     im->vfinger_invert_x = false;
@@ -247,13 +203,12 @@ set_device_clipboard(struct sc_input_manager *im, bool paste,
 }
 
 static void
-set_screen_power_mode(struct sc_input_manager *im,
-                      enum sc_screen_power_mode mode) {
+set_display_power(struct sc_input_manager *im, bool on) {
     assert(im->controller);
 
     struct sc_control_msg msg;
-    msg.type = SC_CONTROL_MSG_TYPE_SET_SCREEN_POWER_MODE;
-    msg.set_screen_power_mode.mode = mode;
+    msg.type = SC_CONTROL_MSG_TYPE_SET_DISPLAY_POWER;
+    msg.set_display_power.on = on;
 
     if (!sc_controller_push_msg(im->controller, &msg)) {
         LOGW("Could not request 'set screen power mode'");
@@ -329,6 +284,18 @@ open_hard_keyboard_settings(struct sc_input_manager *im) {
     }
 }
 
+static void
+reset_video(struct sc_input_manager *im) {
+    assert(im->controller);
+
+    struct sc_control_msg msg;
+    msg.type = SC_CONTROL_MSG_TYPE_RESET_VIDEO;
+
+    if (!sc_controller_push_msg(im->controller, &msg)) {
+        LOGW("Could not request reset video");
+    }
+}
+
 static void
 apply_orientation_transform(struct sc_input_manager *im,
                             enum sc_orientation transform) {
@@ -346,7 +313,8 @@ sc_input_manager_process_text_input(struct sc_input_manager *im,
         return;
     }
 
-    if (is_shortcut_mod(im, SDL_GetModState())) {
+    if (sc_shortcut_mods_is_shortcut_mod(im->sdl_shortcut_mods,
+                                         SDL_GetModState())) {
         // A shortcut must never generate text events
         return;
     }
@@ -413,8 +381,9 @@ sc_input_manager_process_key(struct sc_input_manager *im,
     // press/release is a modifier key.
     // The second condition is necessary to ignore the release of the modifier
     // key (because in this case mod is 0).
-    bool is_shortcut = is_shortcut_mod(im, mod)
-                    || is_shortcut_key(im, sdl_keycode);
+    uint16_t mods = im->sdl_shortcut_mods;
+    bool is_shortcut = sc_shortcut_mods_is_shortcut_mod(mods, mod)
+                    || sc_shortcut_mods_is_shortcut_key(mods, sdl_keycode);
 
     if (down && !repeat) {
         if (sdl_keycode == im->last_keycode && mod == im->last_mod) {
@@ -457,10 +426,8 @@ sc_input_manager_process_key(struct sc_input_manager *im,
                 return;
             case SDLK_o:
                 if (control && !repeat && down && !paused) {
-                    enum sc_screen_power_mode mode = shift
-                                                   ? SC_SCREEN_POWER_MODE_NORMAL
-                                                   : SC_SCREEN_POWER_MODE_OFF;
-                    set_screen_power_mode(im, mode);
+                    bool on = shift;
+                    set_display_power(im, on);
                 }
                 return;
             case SDLK_z:
@@ -536,7 +503,7 @@ sc_input_manager_process_key(struct sc_input_manager *im,
                 return;
             case SDLK_f:
                 if (video && !shift && !repeat && down) {
-                    sc_screen_switch_fullscreen(im->screen);
+                    sc_screen_toggle_fullscreen(im->screen);
                 }
                 return;
             case SDLK_w:
@@ -566,8 +533,12 @@ sc_input_manager_process_key(struct sc_input_manager *im,
                 }
                 return;
             case SDLK_r:
-                if (control && !shift && !repeat && down && !paused) {
-                    rotate_device(im);
+                if (control && !repeat && down && !paused) {
+                    if (shift) {
+                        reset_video(im);
+                    } else {
+                        rotate_device(im);
+                    }
                 }
                 return;
             case SDLK_k:
@@ -836,7 +807,7 @@ sc_input_manager_process_mouse_button(struct sc_input_manager *im,
     }
 
     bool change_vfinger = event->button == SDL_BUTTON_LEFT &&
-            ((down && !im->vfinger_down && (ctrl_pressed ^ shift_pressed)) ||
+            ((down && !im->vfinger_down && (ctrl_pressed || shift_pressed)) ||
              (!down && im->vfinger_down));
     bool use_finger = im->vfinger_down || change_vfinger;
 
@@ -868,16 +839,28 @@ sc_input_manager_process_mouse_button(struct sc_input_manager *im,
     // In other words, the center of the rotation/scaling is the center of the
     // screen.
     //
-    // To simulate a tilt gesture (a vertical slide with two fingers), Shift
-    // can be used instead of Ctrl. The "virtual finger" has a position
+    // To simulate a vertical tilt gesture (a vertical slide with two fingers),
+    // Shift can be used instead of Ctrl. The "virtual finger" has a position
     // inverted with respect to the vertical axis of symmetry in the middle of
     // the screen.
+    //
+    // To simulate a horizontal tilt gesture (a horizontal slide with two
+    // fingers), Ctrl+Shift can be used. The "virtual finger" has a position
+    // inverted with respect to the horizontal axis of symmetry in the middle
+    // of the screen. It is expected to be less frequently used, that's why the
+    // one-mod shortcuts are assigned to rotation and vertical tilt.
     if (change_vfinger) {
         struct sc_point mouse =
             sc_screen_convert_window_to_frame_coords(im->screen, event->x,
                                                                  event->y);
         if (down) {
-            im->vfinger_invert_x = ctrl_pressed || shift_pressed;
+            // Ctrl  Shift     invert_x  invert_y
+            // ----  ----- ==> --------  --------
+            //   0     0           0         0      -
+            //   0     1           1         0      vertical tilt
+            //   1     0           1         1      rotate
+            //   1     1           0         1      horizontal tilt
+            im->vfinger_invert_x = ctrl_pressed ^ shift_pressed;
             im->vfinger_invert_y = ctrl_pressed;
         }
         struct sc_point vfinger = inverse_point(mouse, im->screen->frame_size,
