@@ -66,56 +66,6 @@ get_server_path(void) {
     return server_path;
 }
 
-static void
-sc_server_params_destroy(struct sc_server_params *params) {
-    // The server stores a copy of the params provided by the user
-    free((char *) params->req_serial);
-    free((char *) params->crop);
-    free((char *) params->video_codec_options);
-    free((char *) params->audio_codec_options);
-    free((char *) params->video_encoder);
-    free((char *) params->audio_encoder);
-    free((char *) params->tcpip_dst);
-    free((char *) params->camera_id);
-    free((char *) params->camera_ar);
-}
-
-static bool
-sc_server_params_copy(struct sc_server_params *dst,
-                      const struct sc_server_params *src) {
-    *dst = *src;
-
-    // The params reference user-allocated memory, so we must copy them to
-    // handle them from another thread
-
-#define COPY(FIELD) do { \
-    dst->FIELD = NULL; \
-    if (src->FIELD) { \
-        dst->FIELD = strdup(src->FIELD); \
-        if (!dst->FIELD) { \
-            goto error; \
-        } \
-    } \
-} while(0)
-
-    COPY(req_serial);
-    COPY(crop);
-    COPY(video_codec_options);
-    COPY(audio_codec_options);
-    COPY(video_encoder);
-    COPY(audio_encoder);
-    COPY(tcpip_dst);
-    COPY(camera_id);
-    COPY(camera_ar);
-#undef COPY
-
-    return true;
-
-error:
-    sc_server_params_destroy(dst);
-    return false;
-}
-
 static bool
 push_server(struct sc_intr *intr, const char *serial) {
     char *server_path = get_server_path();
@@ -147,7 +97,7 @@ log_level_to_server_string(enum sc_log_level level) {
             return "error";
         default:
             assert(!"unexpected log level");
-            return "(unknown)";
+            return NULL;
     }
 }
 
@@ -183,6 +133,7 @@ sc_server_get_codec_name(enum sc_codec codec) {
         case SC_CODEC_RAW:
             return "raw";
         default:
+            assert(!"unexpected codec");
             return NULL;
     }
 }
@@ -197,10 +148,41 @@ sc_server_get_camera_facing_name(enum sc_camera_facing camera_facing) {
         case SC_CAMERA_FACING_EXTERNAL:
             return "external";
         default:
+            assert(!"unexpected camera facing");
             return NULL;
     }
 }
 
+static const char *
+sc_server_get_audio_source_name(enum sc_audio_source audio_source) {
+    switch (audio_source) {
+        case SC_AUDIO_SOURCE_OUTPUT:
+            return "output";
+        case SC_AUDIO_SOURCE_MIC:
+            return "mic";
+        case SC_AUDIO_SOURCE_PLAYBACK:
+            return "playback";
+        default:
+            assert(!"unexpected audio source");
+            return NULL;
+    }
+}
+
+static bool
+validate_string(const char *s) {
+    // The parameters values are passed as command line arguments to adb, so
+    // they must either be properly escaped, or they must not contain any
+    // special shell characters.
+    // Since they are not properly escaped on Windows anyway (see
+    // sys/win/process.c), just forbid special shell characters.
+    if (strpbrk(s, " ;'\"*$?&`#\\|<>[]{}()!~\r\n")) {
+        LOGE("Invalid server param: [%s]", s);
+        return false;
+    }
+
+    return true;
+}
+
 static sc_pid
 execute_server(struct sc_server *server,
                const struct sc_server_params *params) {
@@ -219,18 +201,31 @@ execute_server(struct sc_server *server,
     cmd[count++] = "app_process";
 
 #ifdef SERVER_DEBUGGER
+    uint16_t sdk_version = sc_adb_get_device_sdk_version(&server->intr, serial);
+    if (!sdk_version) {
+        LOGE("Could not determine SDK version");
+        return 0;
+    }
+
 # define SERVER_DEBUGGER_PORT "5005"
-    cmd[count++] =
-# ifdef SERVER_DEBUGGER_METHOD_NEW
-        /* Android 9 and above */
-        "-XjdwpProvider:internal -XjdwpOptions:transport=dt_socket,suspend=y,"
-        "server=y,address="
-# else
-        /* Android 8 and below */
-        "-agentlib:jdwp=transport=dt_socket,suspend=y,server=y,address="
-# endif
-            SERVER_DEBUGGER_PORT;
+    const char *dbg;
+    if (sdk_version < 28) {
+        // Android < 9
+        dbg = "-agentlib:jdwp=transport=dt_socket,suspend=y,server=y,address="
+              SERVER_DEBUGGER_PORT;
+    } else if (sdk_version < 30) {
+        // Android >= 9 && Android < 11
+        dbg = "-XjdwpProvider:internal -XjdwpOptions:transport=dt_socket,"
+              "suspend=y,server=y,address=" SERVER_DEBUGGER_PORT;
+    } else {
+        // Android >= 11
+        // Contrary to the other methods, this does not suspend on start.
+        // <https://github.com/Genymobile/scrcpy/pull/5466>
+        dbg = "-XjdwpProvider:adbconnection";
+    }
+    cmd[count++] = dbg;
 #endif
+
     cmd[count++] = "/"; // unused
     cmd[count++] = "com.genymobile.scrcpy.Server";
     cmd[count++] = SCRCPY_VERSION;
@@ -243,6 +238,11 @@ execute_server(struct sc_server *server,
         } \
         cmd[count++] = p; \
     } while(0)
+#define VALIDATE_STRING(s) do { \
+        if (!validate_string(s)) { \
+            goto end; \
+        } \
+    } while(0)
 
     ADD_PARAM("scid=%08x", params->scid);
     ADD_PARAM("log_level=%s", log_level_to_server_string(params->log_level));
@@ -271,23 +271,43 @@ execute_server(struct sc_server *server,
         assert(params->video_source == SC_VIDEO_SOURCE_CAMERA);
         ADD_PARAM("video_source=camera");
     }
-    if (params->audio_source == SC_AUDIO_SOURCE_MIC) {
-        ADD_PARAM("audio_source=mic");
+    // If audio is enabled, an "auto" audio source must have been resolved
+    assert(params->audio_source != SC_AUDIO_SOURCE_AUTO || !params->audio);
+    if (params->audio_source != SC_AUDIO_SOURCE_OUTPUT && params->audio) {
+        ADD_PARAM("audio_source=%s",
+                  sc_server_get_audio_source_name(params->audio_source));
+    }
+    if (params->audio_dup) {
+        ADD_PARAM("audio_dup=true");
     }
     if (params->max_size) {
         ADD_PARAM("max_size=%" PRIu16, params->max_size);
     }
     if (params->max_fps) {
-        ADD_PARAM("max_fps=%" PRIu16, params->max_fps);
+        VALIDATE_STRING(params->max_fps);
+        ADD_PARAM("max_fps=%s", params->max_fps);
     }
-    if (params->lock_video_orientation != SC_LOCK_VIDEO_ORIENTATION_UNLOCKED) {
-        ADD_PARAM("lock_video_orientation=%" PRIi8,
-                  params->lock_video_orientation);
+    if (params->angle) {
+        VALIDATE_STRING(params->angle);
+        ADD_PARAM("angle=%s", params->angle);
+    }
+    if (params->capture_orientation_lock != SC_ORIENTATION_UNLOCKED
+            || params->capture_orientation != SC_ORIENTATION_0) {
+        if (params->capture_orientation_lock == SC_ORIENTATION_LOCKED_INITIAL) {
+            ADD_PARAM("capture_orientation=@");
+        } else {
+            const char *orient =
+                sc_orientation_get_name(params->capture_orientation);
+            bool locked =
+                params->capture_orientation_lock != SC_ORIENTATION_UNLOCKED;
+            ADD_PARAM("capture_orientation=%s%s", locked ? "@" : "", orient);
+        }
     }
     if (server->tunnel.forward) {
         ADD_PARAM("tunnel_forward=true");
     }
     if (params->crop) {
+        VALIDATE_STRING(params->crop);
         ADD_PARAM("crop=%s", params->crop);
     }
     if (!params->control) {
@@ -298,9 +318,11 @@ execute_server(struct sc_server *server,
         ADD_PARAM("display_id=%" PRIu32, params->display_id);
     }
     if (params->camera_id) {
+        VALIDATE_STRING(params->camera_id);
         ADD_PARAM("camera_id=%s", params->camera_id);
     }
     if (params->camera_size) {
+        VALIDATE_STRING(params->camera_size);
         ADD_PARAM("camera_size=%s", params->camera_size);
     }
     if (params->camera_facing != SC_CAMERA_FACING_ANY) {
@@ -308,6 +330,7 @@ execute_server(struct sc_server *server,
             sc_server_get_camera_facing_name(params->camera_facing));
     }
     if (params->camera_ar) {
+        VALIDATE_STRING(params->camera_ar);
         ADD_PARAM("camera_ar=%s", params->camera_ar);
     }
     if (params->camera_fps) {
@@ -322,16 +345,25 @@ execute_server(struct sc_server *server,
     if (params->stay_awake) {
         ADD_PARAM("stay_awake=true");
     }
+    if (params->screen_off_timeout != -1) {
+        assert(params->screen_off_timeout >= 0);
+        uint64_t ms = SC_TICK_TO_MS(params->screen_off_timeout);
+        ADD_PARAM("screen_off_timeout=%" PRIu64, ms);
+    }
     if (params->video_codec_options) {
+        VALIDATE_STRING(params->video_codec_options);
         ADD_PARAM("video_codec_options=%s", params->video_codec_options);
     }
     if (params->audio_codec_options) {
+        VALIDATE_STRING(params->audio_codec_options);
         ADD_PARAM("audio_codec_options=%s", params->audio_codec_options);
     }
     if (params->video_encoder) {
+        VALIDATE_STRING(params->video_encoder);
         ADD_PARAM("video_encoder=%s", params->video_encoder);
     }
     if (params->audio_encoder) {
+        VALIDATE_STRING(params->audio_encoder);
         ADD_PARAM("audio_encoder=%s", params->audio_encoder);
     }
     if (params->power_off_on_close) {
@@ -353,6 +385,13 @@ execute_server(struct sc_server *server,
         // By default, power_on is true
         ADD_PARAM("power_on=false");
     }
+    if (params->new_display) {
+        VALIDATE_STRING(params->new_display);
+        ADD_PARAM("new_display=%s", params->new_display);
+    }
+    if (!params->vd_system_decorations) {
+        ADD_PARAM("vd_system_decorations=false");
+    }
     if (params->list & SC_OPTION_LIST_ENCODERS) {
         ADD_PARAM("list_encoders=true");
     }
@@ -365,16 +404,23 @@ execute_server(struct sc_server *server,
     if (params->list & SC_OPTION_LIST_CAMERA_SIZES) {
         ADD_PARAM("list_camera_sizes=true");
     }
+    if (params->list & SC_OPTION_LIST_APPS) {
+        ADD_PARAM("list_apps=true");
+    }
 
 #undef ADD_PARAM
 
     cmd[count++] = NULL;
 
 #ifdef SERVER_DEBUGGER
-    LOGI("Server debugger waiting for a client on device port "
-         SERVER_DEBUGGER_PORT "...");
-    // From the computer, run
-    //     adb forward tcp:5005 tcp:5005
+    LOGI("Server debugger listening%s...",
+         sdk_version < 30 ? " on port " SERVER_DEBUGGER_PORT : "");
+    // For Android < 11, from the computer:
+    //     - run `adb forward tcp:5005 tcp:5005`
+    // For Android >= 11:
+    //     - execute `adb jdwp` to get the jdwp port
+    //     - run `adb forward tcp:5005 jdwp:XXXX` (replace XXXX)
+    //
     // Then, from Android Studio: Run > Debug > Edit configurations...
     // On the left, click on '+', "Remote", with:
     //     Host: localhost
@@ -447,22 +493,18 @@ connect_to_server(struct sc_server *server, unsigned attempts, sc_tick delay,
 bool
 sc_server_init(struct sc_server *server, const struct sc_server_params *params,
               const struct sc_server_callbacks *cbs, void *cbs_userdata) {
-    bool ok = sc_server_params_copy(&server->params, params);
-    if (!ok) {
-        LOG_OOM();
-        return false;
-    }
+    // The allocated data in params (const char *) must remain valid until the
+    // end of the program
+    server->params = *params;
 
-    ok = sc_mutex_init(&server->mutex);
+    bool ok = sc_mutex_init(&server->mutex);
     if (!ok) {
-        sc_server_params_destroy(&server->params);
         return false;
     }
 
     ok = sc_cond_init(&server->cond_stopped);
     if (!ok) {
         sc_mutex_destroy(&server->mutex);
-        sc_server_params_destroy(&server->params);
         return false;
     }
 
@@ -470,7 +512,6 @@ sc_server_init(struct sc_server *server, const struct sc_server_params *params,
     if (!ok) {
         sc_cond_destroy(&server->cond_stopped);
         sc_mutex_destroy(&server->mutex);
-        sc_server_params_destroy(&server->params);
         return false;
     }
 
@@ -607,6 +648,14 @@ sc_server_connect_to(struct sc_server *server, struct sc_server_info *info) {
         }
     }
 
+    if (control_socket != SC_SOCKET_NONE) {
+        // Disable Nagle's algorithm for the control socket
+        // (it only impacts the sending side, so it is useless to set it
+        // for the other sockets)
+        bool ok = net_set_tcp_nodelay(control_socket, true);
+        (void) ok; // error already logged
+    }
+
     // we don't need the adb tunnel anymore
     sc_adb_tunnel_close(tunnel, &server->intr, serial,
                         server->device_socket_name);
@@ -1101,7 +1150,6 @@ sc_server_destroy(struct sc_server *server) {
 
     free(server->serial);
     free(server->device_socket_name);
-    sc_server_params_destroy(&server->params);
     sc_intr_destroy(&server->intr);
     sc_cond_destroy(&server->cond_stopped);
     sc_mutex_destroy(&server->mutex);
