@@ -2,6 +2,9 @@ package com.genymobile.scrcpy;
 
 import com.genymobile.scrcpy.audio.AudioCodec;
 import com.genymobile.scrcpy.audio.AudioSource;
+import com.genymobile.scrcpy.device.Device;
+import com.genymobile.scrcpy.device.NewDisplay;
+import com.genymobile.scrcpy.device.Orientation;
 import com.genymobile.scrcpy.device.Size;
 import com.genymobile.scrcpy.util.CodecOption;
 import com.genymobile.scrcpy.util.Ln;
@@ -11,6 +14,7 @@ import com.genymobile.scrcpy.video.VideoCodec;
 import com.genymobile.scrcpy.video.VideoSource;
 
 import android.graphics.Rect;
+import android.util.Pair;
 
 import java.util.List;
 import java.util.Locale;
@@ -30,7 +34,7 @@ public class Options {
     private int videoBitRate = 8000000;
     private int audioBitRate = 128000;
     private float maxFps;
-    private int lockVideoOrientation = -1;
+    private float angle;
     private boolean tunnelForward;
     private Rect crop;
     private boolean control = true;
@@ -43,6 +47,7 @@ public class Options {
     private boolean cameraHighSpeed;
     private boolean showTouches;
     private boolean stayAwake;
+    private int screenOffTimeout = -1;
     private List<CodecOption> videoCodecOptions;
     private List<CodecOption> audioCodecOptions;
 
@@ -54,10 +59,17 @@ public class Options {
     private boolean cleanup = true;
     private boolean powerOn = true;
 
+    private NewDisplay newDisplay;
+    private boolean vdSystemDecorations = true;
+
+    private Orientation.Lock captureOrientationLock = Orientation.Lock.Unlocked;
+    private Orientation captureOrientation = Orientation.Orient0;
+
     private boolean listEncoders;
     private boolean listDisplays;
     private boolean listCameras;
     private boolean listCameraSizes;
+    private boolean listApps;
 
     // Options not used by the scrcpy client, but useful to use scrcpy-server directly
     private boolean sendDeviceMeta = true; // send device name and size
@@ -117,8 +129,8 @@ public class Options {
         return maxFps;
     }
 
-    public int getLockVideoOrientation() {
-        return lockVideoOrientation;
+    public float getAngle() {
+        return angle;
     }
 
     public boolean isTunnelForward() {
@@ -169,6 +181,10 @@ public class Options {
         return stayAwake;
     }
 
+    public int getScreenOffTimeout() {
+        return screenOffTimeout;
+    }
+
     public List<CodecOption> getVideoCodecOptions() {
         return videoCodecOptions;
     }
@@ -205,8 +221,24 @@ public class Options {
         return powerOn;
     }
 
+    public NewDisplay getNewDisplay() {
+        return newDisplay;
+    }
+
+    public Orientation getCaptureOrientation() {
+        return captureOrientation;
+    }
+
+    public Orientation.Lock getCaptureOrientationLock() {
+        return captureOrientationLock;
+    }
+
+    public boolean getVDSystemDecorations() {
+        return vdSystemDecorations;
+    }
+
     public boolean getList() {
-        return listEncoders || listDisplays || listCameras || listCameraSizes;
+        return listEncoders || listDisplays || listCameras || listCameraSizes || listApps;
     }
 
     public boolean getListEncoders() {
@@ -225,6 +257,10 @@ public class Options {
         return listCameraSizes;
     }
 
+    public boolean getListApps() {
+        return listApps;
+    }
+
     public boolean getSendDeviceMeta() {
         return sendDeviceMeta;
     }
@@ -323,8 +359,8 @@ public class Options {
                 case "max_fps":
                     options.maxFps = parseFloat("max_fps", value);
                     break;
-                case "lock_video_orientation":
-                    options.lockVideoOrientation = Integer.parseInt(value);
+                case "angle":
+                    options.angle = parseFloat("angle", value);
                     break;
                 case "tunnel_forward":
                     options.tunnelForward = Boolean.parseBoolean(value);
@@ -346,6 +382,12 @@ public class Options {
                 case "stay_awake":
                     options.stayAwake = Boolean.parseBoolean(value);
                     break;
+                case "screen_off_timeout":
+                    options.screenOffTimeout = Integer.parseInt(value);
+                    if (options.screenOffTimeout < -1) {
+                        throw new IllegalArgumentException("Invalid screen off timeout: " + options.screenOffTimeout);
+                    }
+                    break;
                 case "video_codec_options":
                     options.videoCodecOptions = CodecOption.parse(value);
                     break;
@@ -388,6 +430,9 @@ public class Options {
                 case "list_camera_sizes":
                     options.listCameraSizes = Boolean.parseBoolean(value);
                     break;
+                case "list_apps":
+                    options.listApps = Boolean.parseBoolean(value);
+                    break;
                 case "camera_id":
                     if (!value.isEmpty()) {
                         options.cameraId = value;
@@ -418,6 +463,17 @@ public class Options {
                 case "camera_high_speed":
                     options.cameraHighSpeed = Boolean.parseBoolean(value);
                     break;
+                case "new_display":
+                    options.newDisplay = parseNewDisplay(value);
+                    break;
+                case "vd_system_decorations":
+                    options.vdSystemDecorations = Boolean.parseBoolean(value);
+                    break;
+                case "capture_orientation":
+                    Pair<Orientation.Lock, Orientation> pair = parseCaptureOrientation(value);
+                    options.captureOrientationLock = pair.first;
+                    options.captureOrientation = pair.second;
+                    break;
                 case "send_device_meta":
                     options.sendDeviceMeta = Boolean.parseBoolean(value);
                     break;
@@ -445,6 +501,11 @@ public class Options {
             }
         }
 
+        if (options.newDisplay != null) {
+            assert options.displayId == 0 : "Must not set both displayId and newDisplay";
+            options.displayId = Device.DISPLAY_ID_NONE;
+        }
+
         return options;
     }
 
@@ -475,6 +536,9 @@ public class Options {
         }
         int width = Integer.parseInt(tokens[0]);
         int height = Integer.parseInt(tokens[1]);
+        if (width <= 0 || height <= 0) {
+            throw new IllegalArgumentException("Invalid non-positive size dimension: \"" + size + "\"");
+        }
         return new Size(width, height);
     }
 
@@ -501,4 +565,57 @@ public class Options {
             throw new IllegalArgumentException("Invalid float value for " + key + ": \"" + value + "\"");
         }
     }
+
+    private static NewDisplay parseNewDisplay(String newDisplay) {
+        // Possible inputs:
+        //  - "" (empty string)
+        //  - "<width>x<height>/<dpi>"
+        //  - "<width>x<height>"
+        //  - "/<dpi>"
+        if (newDisplay.isEmpty()) {
+            return new NewDisplay();
+        }
+
+        String[] tokens = newDisplay.split("/");
+
+        Size size;
+        if (!tokens[0].isEmpty()) {
+            size = parseSize(tokens[0]);
+        } else {
+            size = null;
+        }
+
+        int dpi;
+        if (tokens.length >= 2) {
+            dpi = Integer.parseInt(tokens[1]);
+            if (dpi <= 0) {
+                throw new IllegalArgumentException("Invalid non-positive dpi: " + tokens[1]);
+            }
+        } else {
+            dpi = 0;
+        }
+
+        return new NewDisplay(size, dpi);
+    }
+
+    private static Pair<Orientation.Lock, Orientation> parseCaptureOrientation(String value) {
+        if (value.isEmpty()) {
+            throw new IllegalArgumentException("Empty capture orientation string");
+        }
+
+        Orientation.Lock lock;
+        if (value.charAt(0) == '@') {
+            // Consume '@'
+            value = value.substring(1);
+            if (value.isEmpty()) {
+                // Only '@': lock to the initial orientation (orientation is unused)
+                return Pair.create(Orientation.Lock.LockedInitial, Orientation.Orient0);
+            }
+            lock = Orientation.Lock.LockedValue;
+        } else {
+            lock = Orientation.Lock.Unlocked;
+        }
+
+        return Pair.create(lock, Orientation.getByName(value));
+    }
 }
